@@ -8,7 +8,7 @@
 #  Path to R installation on worker.
 #  \dQuote{} means R installation on the PATH is used.
 # @param r.options [\code{character}]
-#   Options for R and Rscript, one option per element of the vector, 
+#   Options for R and Rscript, one option per element of the vector,
 #   a la \dQuote{--vanilla}.
 # @param script [\code{character(1)}]\cr
 #   Path to helper script on worker.
@@ -26,10 +26,12 @@
 # @param classes [\code{character}]\cr
 #   Extra classes, more specific than dQuote{Worker}.
 #   Will be added to the class attribute of the object.
+# @param nice [\code{integer(1)}]\cr
+#   Process priority to run R with set via nice. Integers between -20 and 19 are allowed.
+#   If missing, processes are not nice'd and the system default applies (usually 0).
 # @return [\code{\link{Worker}}].
-makeWorker = function(ssh, nodename, rhome, r.options=c("--no-save", "--no-restore", "--no-init-file", "--no-site-file"), 
-  script, ncpus, max.jobs, max.load, classes) {
-  
+makeWorker = function(ssh, nodename, rhome, r.options=c("--no-save", "--no-restore", "--no-init-file", "--no-site-file"),
+  script, ncpus, max.jobs, max.load, nice, classes) {
   checkArg(ssh, "logical", len=1L, na.ok=FALSE)
   checkArg(nodename, "character", len=1L, na.ok=FALSE)
   checkArg(rhome, "character", len=1L, na.ok=FALSE)
@@ -75,10 +77,17 @@ makeWorker = function(ssh, nodename, rhome, r.options=c("--no-save", "--no-resto
     if (max.load > ncpus)
       stopf("max.load must be <= ncpus = %i!", ncpus)
   }
+  if (missing(nice)) {
+    nice = ""
+  } else {
+    nice = convertInteger(nice)
+    checkArg(nice, "integer", len=1L, na.ok=FALSE, lower=-20, upper=19)
+  }
 
   w$ncpus = ncpus
   w$max.jobs = max.jobs
   w$max.load = max.load
+  w$nice = nice
   return(w)
 }
 
