@@ -27,9 +27,11 @@
 #' # now reduce one final time on master
 #' reduceResults(reg, fun=function(aggr,job,res) f(aggr, res))
 batchReduce = function(reg, fun, xs, init, block.size, more.args=list()) {
-  checkArg(reg, cl="Registry")
+  checkRegistry(reg)
+  syncRegistry(reg)
   checkArg(fun, formals=c("aggr", "x"))
-  checkArg(xs, cl="vector")
+  if (!is.vector(xs))
+    stop("Argument xs must be a vector")
   block.size = convertInteger(block.size)
   checkArg(block.size, "integer", len=1L, lower=1L, na.ok=FALSE)
   if (dbGetJobCount(reg) > 0L)
@@ -42,6 +44,6 @@ batchReduce = function(reg, fun, xs, init, block.size, more.args=list()) {
 
 batchReduceWrapper = function(xs.block, .fun, .init, ...) {
   fun = function(aggr, x)
-    .fun(aggr, x, ...)    
+    .fun(aggr, x, ...)
   Reduce(fun, xs.block, init=.init)
-}  
+}
