@@ -4,7 +4,7 @@ writeRscripts = function(reg, ids, resources.timestamp, disable.mail, delays, in
       "Sys.sleep(%%f)",
       "options(BatchJobs.on.slave=TRUE, BatchJobs.resources.path='%s')",
       "library(BatchJobs)",
-      "BatchJobs:::doJob(",
+      "res = BatchJobs:::doJob(",
       "\treg=loadRegistry('%s'),",
       "\tids=c(%%s),",
       "\tmultiple.result.files=%s,",
@@ -18,7 +18,7 @@ writeRscripts = function(reg, ids, resources.timestamp, disable.mail, delays, in
     template = paste(
       "ignore = %%f",
       "setOnSlave(TRUE, resources.path='%s')",
-      "doJob(",
+      "res = doJob(",
       "\treg=loadRegistry('%s'),",
       "\tids=c(%%s),",
       "\tmultiple.result.files=%s,",
@@ -29,18 +29,19 @@ writeRscripts = function(reg, ids, resources.timestamp, disable.mail, delays, in
       sep="\n")
   }
 
+
   fids = vapply(ids, head, integer(1L), 1L) # first job id in chunk
   first = head(fids, 1L)
   last = tail(fids, 1L)
-  ids.str = vapply(ids, function(id) collapse(paste(id, "L", sep="")), character(1L)) # ids as collapsed strings
+  ids = vapply(ids, function(id) collapse(paste(id, "L", sep="")), character(1L)) # ids as collapsed strings
 
   # print the constant arguments (of length 1) into the template
   resources.path = getResourcesFilePath(reg, resources.timestamp)
   template = sprintf(template, resources.path, reg$file.dir, reg$multiple.result.files, disable.mail, first, last)
 
-  # print delays and ids into template. sprintf will return a string of length length(delays) == length(ids.str) == length(ids)
+  # print delays and ids into template. sprintf will return a string of length length(delays) == length(ids)
   # put this together with file names into an mapply on cat.
   mapply(FUN=cat, SIMPLIFY=FALSE, USE.NAMES=FALSE,
-         sprintf(template, delays, ids.str),
+         sprintf(template, delays, ids),
          file = getRScriptFilePath(fids, reg = reg))
 }
